@@ -1,12 +1,12 @@
 """this module contains some core utility functions that in turn do not depend on anything else in this library"""
 
 import base64
+from datetime import datetime, timezone
 import hashlib
 import json
 import os
-import shutil
-from datetime import datetime
 from pathlib import Path
+import shutil
 from typing import Union
 
 from beartype import beartype
@@ -30,6 +30,107 @@ class PrettyDict(Box):
         self.__repr__()
 
 
+def fix_embedded_newlines_in_csv(path: Union[str, Path]) -> bool:
+    """
+    Detects literal '\\n' sequences in the CSV at `path` and replaces them
+    with real newlines, rewriting the file in-place.
+
+    Returns:
+        True if the file was modified (i.e. a '\\n' was found and fixed),
+        False if no literal '\\n' was present.
+    """
+    p = Path(path)
+    text = p.read_text(encoding="utf-8")
+
+    # If there are no literal "\n" sequences, nothing to do
+    if r"\n" not in text:
+        return False
+
+    # Replace all literal "\n" with actual newlines
+    fixed = text.replace(r"\n", "\n")
+    p.write_text(fixed, encoding="utf-8")
+    return True
+
+
+def elapsed_minutes(
+    start: Union[str, datetime],
+    end: Union[str, datetime],
+) -> int:
+    """
+    Compute the number of minutes elapsed between two timestamps,
+    rounding to the nearest whole minute.
+
+    Parameters:
+        start: ISO-8601 UTC string (e.g. "2025-04-16T18:03:16.154Z")
+               or a timezone-aware datetime.
+        end:   Same format as start.
+
+    Returns:
+        Elapsed time in minutes (int), rounded to the nearest minute.
+    """
+
+    def to_dt(ts):
+        if isinstance(ts, datetime):
+            return ts
+        # parse "YYYY-MM-DDTHH:MM:SS.sssZ"
+        dt = datetime.strptime(ts, "%Y-%m-%dT%H:%M:%S.%fZ")
+        return dt.replace(tzinfo=timezone.utc)
+
+    dt_start = to_dt(start)
+    dt_end = to_dt(end)
+    seconds_elapsed = (dt_end - dt_start).total_seconds()
+    minutes = seconds_elapsed / 60
+    return int(round(minutes))
+
+
+@beartype
+def ensure_file_extension(
+    *,
+    file_paths: list[Union[str, Path]],
+    extension: str,
+) -> list[str]:
+    """
+    Rename each file in file_paths so that it ends with the specified extension,
+    and return the list of resulting file paths. Files already
+    ending with the specified extension (case-insensitive) are left untouched.
+
+    Args:
+        file_paths (List[Union[str, Path]]): List of file paths to process
+        extension (str): The desired file extension (with or without leading dot)
+
+    Returns:
+        List[Path]: List of resulting file paths
+
+    Idempotent: running it again makes no further changes.
+    """
+    # Ensure extension starts with a dot
+    if not extension.startswith("."):
+        extension = f".{extension}"
+
+    resulting_paths = []
+
+    for fp in file_paths:
+        p = Path(fp)
+
+        # Already has the desired extension? leave as is
+        if p.suffix.lower() == extension.lower():
+            resulting_paths.append(str(p))
+            continue
+
+        new_path = p.with_suffix(extension)
+
+        # If the file with new extension already exists, skip renaming—but still return new_path
+        if new_path.exists():
+            resulting_paths.append(str(new_path))
+            continue
+
+        # Otherwise rename and return the new path
+        os.rename(p, new_path)
+        resulting_paths.append(str(new_path))
+
+    return resulting_paths
+
+
 def set_key_to_value(
     obj: dict,
     target_key: str,
@@ -367,8 +468,7 @@ def _ensure_do_folder() -> Path:
     else:
         deeporigin_path = Path.home() / ".deeporigin"
 
-    if not deeporigin_path.exists():
-        deeporigin_path.mkdir(parents=True)
+    deeporigin_path.mkdir(parents=True, exist_ok=True)
 
     return deeporigin_path
 
